from geometry_msgs.msg import Pose, PoseWithCovariance
from mrpt import pymrpt as pymrpt

def CPose3DQuat_to_ROS_Pose_msg(pq: _mrpt.poses.CPose3DQuat) -> Pose: ...
def ROS_Pose_msg_to_CPose3DQuat(p: Pose) -> _mrpt.poses.CPose3DQuat: ...
def CPose3D_to_ROS_Pose_msg(p: _mrpt.poses.CPose3D) -> Pose: ...
def ROS_Pose_msg_to_CPose3D(p: Pose) -> _mrpt.poses.CPose3D: ...
def CPose2D_to_ROS_Pose_msg(q: _mrpt.poses.CPose2D) -> Pose: ...
def ROS_Pose_msg_to_CPose2D(p: Pose) -> _mrpt.poses.CPose2D: ...
def ROS_PoseWithCovariance_msg_to_CPose3DPDFGaussian(p: PoseWithCovariance) -> _mrpt.poses.CPose3DPDFGaussian: ...
def CPose3DPDFGaussian_to_ROS_PoseWithCovariance_msg(p: _mrpt.poses.CPose3DPDFGaussian) -> PoseWithCovariance: ...
