/* +------------------------------------------------------------------------+
   |                     Mobile Robot Programming Toolkit (MRPT)            |
   |                          https://www.mrpt.org/                         |
   |                                                                        |
   | Copyright (c) 2005-2024, Individual contributors, see AUTHORS file     |
   | See: https://www.mrpt.org/Authors - All rights reserved.               |
   | Released under BSD License. See: https://www.mrpt.org/License          |
   +------------------------------------------------------------------------+ */
#pragma once

#include <mrpt/config.h>
#if MRPT_ENABLE_PRECOMPILED_HDRS

// Observations:
#include <mrpt/obs/CAction.h>
#include <mrpt/obs/CObservation.h>

// Very basic classes for maps:
#include <mrpt/maps/CMetricMap.h>
#include <mrpt/serialization/CArchive.h>  // Will be used in all serialized classes (most of them in mrpt-obs)

#endif
