from typing import Any

from typing import overload

class duration_long_std_ratio_1_10000000_t:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, arg0: duration_long_std_ratio_1_10000000_t) -> None: ...
    def assign(self) -> duration_long_std_ratio_1_10000000_t: ...
    @overload
    def count(self) -> int: ...
    @overload
    def count() -> long: ...
    def max(self, *args, **kwargs) -> Any: ...
    def min(self, *args, **kwargs) -> Any: ...
    def post_decrement(self) -> duration_long_std_ratio_1_10000000_t: ...
    def post_increment(self) -> duration_long_std_ratio_1_10000000_t: ...
    def pre_decrement(self) -> duration_long_std_ratio_1_10000000_t: ...
    def pre_increment(self) -> duration_long_std_ratio_1_10000000_t: ...
    def zero(self, *args, **kwargs) -> Any: ...
    def __iadd__(self, __d: duration_long_std_ratio_1_10000000_t) -> duration_long_std_ratio_1_10000000_t: ...
    def __imul__(self, __rhs: int) -> duration_long_std_ratio_1_10000000_t: ...
    def __isub__(self, __d: duration_long_std_ratio_1_10000000_t) -> duration_long_std_ratio_1_10000000_t: ...
    def __itruediv__(self, __rhs: int) -> duration_long_std_ratio_1_10000000_t: ...
    def __neg__(self) -> duration_long_std_ratio_1_10000000_t: ...
    def __pos__(self) -> duration_long_std_ratio_1_10000000_t: ...

class time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t:
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, __dur: duration_long_std_ratio_1_10000000_t) -> None: ...
    @overload
    def __init__(self, arg0: time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t) -> None: ...
    def assign(self) -> time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t: ...
    def max(self, *args, **kwargs) -> Any: ...
    def min(self, *args, **kwargs) -> Any: ...
    def time_since_epoch(self) -> duration_long_std_ratio_1_10000000_t: ...
    def to_double(self) -> float: ...
    def __iadd__(self, __dur: duration_long_std_ratio_1_10000000_t) -> time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t: ...
    def __isub__(self, __dur: duration_long_std_ratio_1_10000000_t) -> time_point_mrpt_Clock_std_chrono_duration_long_std_ratio_1_10000000_t: ...
