from typing import Any

class TEnumType_mrpt_bayes_TKFMethod_t:
    def __init__(self) -> None: ...
    def getBimap(self, *args, **kwargs) -> Any: ...
    def name2value(self, *args, **kwargs) -> Any: ...
    def value2name(self, *args, **kwargs) -> Any: ...

class TEnumType_mrpt_system_VerbosityLevel_t:
    def __init__(self) -> None: ...
    def getBimap(self, *args, **kwargs) -> Any: ...
    def name2value(self, *args, **kwargs) -> Any: ...
    def value2name(self, *args, **kwargs) -> Any: ...
