from typing import overload
import mrpt.pymrpt.mrpt.img
import mrpt.pymrpt.mrpt.obs

@overload
def example2DRangeScan(s: mrpt.pymrpt.mrpt.obs.CObservation2DRangeScan) -> None: ...
@overload
def example2DRangeScan(s: mrpt.pymrpt.mrpt.obs.CObservation2DRangeScan, i: int) -> None: ...
@overload
def example2DRangeScan(classmrpt, int) -> void: ...
@overload
def exampleImage(im: mrpt.pymrpt.mrpt.img.CImage) -> None: ...
@overload
def exampleImage(im: mrpt.pymrpt.mrpt.img.CImage, i: int) -> None: ...
@overload
def exampleImage(classmrpt, int) -> void: ...
