from typing import overload

@overload
def HEIGHTGRIDMAP_EXPORT3D_AS_MESH(value: bool) -> None: ...
@overload
def HEIGHTGRIDMAP_EXPORT3D_AS_MESH(bool) -> void: ...
@overload
def HEIGHTGRIDMAP_EXPORT3D_AS_MESH() -> bool: ...
@overload
def HEIGHTGRIDMAP_EXPORT3D_AS_MESH() -> bool: ...
@overload
def OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL(value: float) -> None: ...
@overload
def OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL(float) -> void: ...
@overload
def OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL() -> float: ...
@overload
def OCTREE_RENDER_MAX_DENSITY_POINTS_PER_SQPIXEL() -> float: ...
@overload
def OCTREE_RENDER_MAX_POINTS_PER_NODE() -> int: ...
@overload
def OCTREE_RENDER_MAX_POINTS_PER_NODE() -> size_t: ...
@overload
def OCTREE_RENDER_MAX_POINTS_PER_NODE(value: int) -> None: ...
@overload
def OCTREE_RENDER_MAX_POINTS_PER_NODE(size_t) -> void: ...
@overload
def USE_SUT_EULER2QUAT_CONVERSION(value: bool) -> None: ...
@overload
def USE_SUT_EULER2QUAT_CONVERSION(bool) -> void: ...
@overload
def USE_SUT_EULER2QUAT_CONVERSION() -> bool: ...
@overload
def USE_SUT_EULER2QUAT_CONVERSION() -> bool: ...
@overload
def USE_SUT_QUAT2EULER_CONVERSION(value: bool) -> None: ...
@overload
def USE_SUT_QUAT2EULER_CONVERSION(bool) -> void: ...
@overload
def USE_SUT_QUAT2EULER_CONVERSION() -> bool: ...
@overload
def USE_SUT_QUAT2EULER_CONVERSION() -> bool: ...
