/* +------------------------------------------------------------------------+
   |                     Mobile Robot Programming Toolkit (MRPT)            |
   |                          https://www.mrpt.org/                         |
   |                                                                        |
   | Copyright (c) 2005-2024, Individual contributors, see AUTHORS file     |
   | See: https://www.mrpt.org/Authors - All rights reserved.               |
   | Released under BSD License. See: https://www.mrpt.org/License          |
   +------------------------------------------------------------------------+ */
#pragma once

#include <mrpt/opengl/Viewport.h>

namespace mrpt::opengl
{
/** Alias for Scene for backwards compatibility with MRPT <2.7.0
 * \ingroup mrpt_opengl_grp
 */
using COpenGLViewport = Viewport;

}  // namespace mrpt::opengl