R"XXX(#version 300 es

// FRAGMENT SHADER: Default shader for MRPT CRenderizable objects
// Jose Luis Blanco Claraco (C) 2019-2023
// Part of the MRPT project

void main()
{
    // Already done automatically:
    //gl_FragDepth = gl_FragCoord.z;
}
)XXX"
