from typing import overload
import mrpt.pymrpt.mrpt.io

def compress(inData: capsule, inDataSize: int, out: mrpt.pymrpt.mrpt.io.CStream) -> None: ...
@overload
def decompress(inData: capsule, inDataSize: int, outData: capsule, outDataBufferSize: int) -> int: ...
@overload
def decompress(inStream: mrpt.pymrpt.mrpt.io.CStream, inDataSize: int, outData: capsule, outDataBufferSize: int) -> int: ...
