from typing import ClassVar

CDGPS: nv_position_type_t
DOPPLER_VELOCITY: nv_position_type_t
FIXEDHEIGHT: nv_position_type_t
FIXEDPOS: nv_position_type_t
FLOATCONV: nv_position_type_t
INS: nv_position_type_t
INS_PSRDIFF: nv_position_type_t
INS_PSRSP: nv_position_type_t
INS_RTKFIXED: nv_position_type_t
INS_RTKFLOAT: nv_position_type_t
IONOFREE_FLOAT: nv_position_type_t
L1_FLOAT: nv_position_type_t
L1_INT: nv_position_type_t
NARROWLANE: nv_position_type_t
NARROW_FLOAT: nv_position_type_t
NARROW_INT: nv_position_type_t
NONE: nv_position_type_t
OMNISTAR: nv_position_type_t
OMNISTAR_HP: nv_position_type_t
OMNISTAR_XP: nv_position_type_t
PROPAGATED: nv_position_type_t
PSRDIFF: nv_position_type_t
RTK_DIRECT_INS: nv_position_type_t
Reserved: nv_position_type_t
SINGLE: nv_position_type_t
WAAS: nv_position_type_t
WIDELANE: nv_position_type_t
WIDE_INT: nv_position_type_t

class nv_position_type_t:
    __doc__: ClassVar[str] = ...  # read-only
    __members__: ClassVar[dict] = ...  # read-only
    CDGPS: ClassVar[nv_position_type_t] = ...
    DOPPLER_VELOCITY: ClassVar[nv_position_type_t] = ...
    FIXEDHEIGHT: ClassVar[nv_position_type_t] = ...
    FIXEDPOS: ClassVar[nv_position_type_t] = ...
    FLOATCONV: ClassVar[nv_position_type_t] = ...
    INS: ClassVar[nv_position_type_t] = ...
    INS_PSRDIFF: ClassVar[nv_position_type_t] = ...
    INS_PSRSP: ClassVar[nv_position_type_t] = ...
    INS_RTKFIXED: ClassVar[nv_position_type_t] = ...
    INS_RTKFLOAT: ClassVar[nv_position_type_t] = ...
    IONOFREE_FLOAT: ClassVar[nv_position_type_t] = ...
    L1_FLOAT: ClassVar[nv_position_type_t] = ...
    L1_INT: ClassVar[nv_position_type_t] = ...
    NARROWLANE: ClassVar[nv_position_type_t] = ...
    NARROW_FLOAT: ClassVar[nv_position_type_t] = ...
    NARROW_INT: ClassVar[nv_position_type_t] = ...
    NONE: ClassVar[nv_position_type_t] = ...
    OMNISTAR: ClassVar[nv_position_type_t] = ...
    OMNISTAR_HP: ClassVar[nv_position_type_t] = ...
    OMNISTAR_XP: ClassVar[nv_position_type_t] = ...
    PROPAGATED: ClassVar[nv_position_type_t] = ...
    PSRDIFF: ClassVar[nv_position_type_t] = ...
    RTK_DIRECT_INS: ClassVar[nv_position_type_t] = ...
    Reserved: ClassVar[nv_position_type_t] = ...
    SINGLE: ClassVar[nv_position_type_t] = ...
    WAAS: ClassVar[nv_position_type_t] = ...
    WIDELANE: ClassVar[nv_position_type_t] = ...
    WIDE_INT: ClassVar[nv_position_type_t] = ...
    __entries: ClassVar[dict] = ...
    def __init__(self, value: int) -> None: ...
    def __and__(self, other: object) -> object: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __getstate__(self) -> int: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __index__(self) -> int: ...
    def __int__(self) -> int: ...
    def __invert__(self) -> object: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __or__(self, other: object) -> object: ...
    def __rand__(self, other: object) -> object: ...
    def __ror__(self, other: object) -> object: ...
    def __rxor__(self, other: object) -> object: ...
    def __setstate__(self, state: int) -> None: ...
    def __xor__(self, other: object) -> object: ...
    @property
    def name(self) -> str: ...
    @property
    def value(self) -> int: ...

def enum2str(val: int) -> str: ...
