from typing import Any

def BumblebeeCamera(*args, **kwargs) -> Any: ...
def CornerXYSimple(*args, **kwargs) -> Any: ...
def CornerXYZ(*args, **kwargs) -> Any: ...
def CornerXYZEye(*args, **kwargs) -> Any: ...
def CornerXYZSimple(*args, **kwargs) -> Any: ...
def Hokuyo_URG(*args, **kwargs) -> Any: ...
def Hokuyo_UTM(*args, **kwargs) -> Any: ...
def Househam_Sprayer(*args, **kwargs) -> Any: ...
def RobotGiraff(*args, **kwargs) -> Any: ...
def RobotPioneer(*args, **kwargs) -> Any: ...
def RobotRhodon(*args, **kwargs) -> Any: ...
