from typing import Any

class EuclideanBase_2U_t:
    def __init__(self) -> None: ...

class EuclideanBase_3U_t:
    def __init__(self) -> None: ...

class Euclidean_2U_t(EuclideanBase_2U_t):
    def __init__(self) -> None: ...

class Euclidean_3U_t(EuclideanBase_3U_t):
    def __init__(self) -> None: ...

class SE_2U_t:
    def __init__(self) -> None: ...
    def asManifoldVector(self, *args, **kwargs) -> Any: ...
    def exp(self, *args, **kwargs) -> Any: ...
    def fromManifoldVector(self, *args, **kwargs) -> Any: ...
    def jacob_dAB_dA(self, *args, **kwargs) -> Any: ...
    def jacob_dAB_dB(self, *args, **kwargs) -> Any: ...
    def jacob_dDexpe_de(self, *args, **kwargs) -> Any: ...
    def log(self, *args, **kwargs) -> Any: ...

class SE_3U_t:
    def __init__(self) -> None: ...
    def asManifoldVector(self, *args, **kwargs) -> Any: ...
    def exp(self, *args, **kwargs) -> Any: ...
    def fromManifoldVector(self, *args, **kwargs) -> Any: ...
    def log(self, *args, **kwargs) -> Any: ...

class SO_2U_t:
    def __init__(self) -> None: ...

class SO_3U_t:
    def __init__(self) -> None: ...
    def exp(self, *args, **kwargs) -> Any: ...
    def fromYPR(self, *args, **kwargs) -> Any: ...
    def log(self, *args, **kwargs) -> Any: ...
    def vee_RmRt(self, *args, **kwargs) -> Any: ...
